#include "KX222.h"

void config_KX222()
{
    I2C1_init();
    I2C1_start();
    
    uint8_t config;
    
    // CNTL1 Register - Put chip in standby mode to write to registers
    I2C1_write(ADDR_W);
    I2C1_write(0x18);
    I2C1_write(0x00);
    I2C1_stop();
    Nop();
    
    // Determine chip config based on max acceleration
    switch (MODE)
    {
        case 32:
            config = 0xF0;
            break;
        
        case 16:
            config = 0xE8;
            break;
            
        case 8:
            config = 0xE0;
            break;
            
        default:
            config = 0xF0;
    }
    
    // ODCNTL Register - Set output data rate to 800Hz
    I2C1_restart();
    I2C1_write(ADDR_W);
    I2C1_write(0x1B);
    I2C1_write(0x06);
    
    // INC5 and INC6 Registers - Set up data ready interrupt on INT2 pin
    I2C1_restart();
    I2C1_write(ADDR_W);
    I2C1_write(0x20);
    I2C1_write(0x3A);
    I2C1_write(0x10);
    
    // CNTL1 Register - Put chip in operating mode and set up parameters
    I2C1_restart();
    I2C1_write(ADDR_W);
    I2C1_write(0x18);
    I2C1_write(config);
    I2C1_stop();
    
}


void read_KX222(Data_KX* data)
{
    uint16_t x_lsb = 0;
    uint16_t x_msb = 0;
    uint16_t y_lsb = 0;
    uint16_t y_msb = 0;
    uint16_t z_lsb = 0;
    uint16_t z_msb = 0;
    int16_t x_accel = 0;
    int16_t y_accel = 0;
    int16_t z_accel = 0;
    float max_value = 32768;
    
    // Start reading acceleration data
    I2C1_start();
    I2C1_write(ADDR_W);
    I2C1_write(0x06);
    
    I2C1_restart();
    I2C1_write(ADDR_R);
    x_lsb = I2C1_read(0);
    x_msb = I2C1_read(0);
    y_lsb = I2C1_read(0);
    y_msb = I2C1_read(0);
    z_lsb = I2C1_read(0);
    z_msb = I2C1_read(1);
    I2C1_stop();
    
    // Combine msb and lsb for each reading
    x_accel = (x_msb << 8) | x_lsb;
    y_accel = (y_msb << 8) | y_lsb;
    z_accel = (z_msb << 8) | z_lsb;
    
    // Store acceleration values in data struct
    data->x_data = (float)MODE*((float)x_accel / max_value);
    data->y_data = (float)MODE*((float)y_accel / max_value);
    data->z_data = (float)MODE*((float)z_accel / max_value);
}

