/* 
 * File:   SD_interface.c
 * Author: zacharykowalczyk
 *
 * NOTE: This is the main way to interface with the SD card
 * It acts as a wrapper for the other SD card functions in ff.c and mmcPIC32.c
 */



#include "SD_interface.h"

//Work registers for fs command
DWORD acc_size;			/* Work register for fs command */
WORD acc_files, acc_dirs;
FILINFO Finfo;
const BYTE ft[] = {0,12,16,32};

//File system object
FATFS Fatfs;
FATFS *fs;      /* Pointer to file system object */
FIL file1;		/* File objects */

/*
 * This function mounts the file system and opens the save file for writing
 */
void SD_init() {
    
    // Set up DEBUGLED1 to turn on when setting up the SD card
    DEBUG1_TRIS = OUTPUT;
    DEBUG1 = DEBUG_ON;
    DEBUG2_TRIS = OUTPUT;
    
    //Initialize Disk
    disk_initialize(0);

    //Mount Filesystem
    f_mount(0, &Fatfs);
    //Open/create Fileset.dex
    FRESULT FOpenRes;
    long accessmode;
    
    // Allocate space for filename
    char basename[] = "Flight";
    char extension[] = ".csv";
    //char filename[] = "Flight.csv";
    char filename[200];
    
    // Figure out a file that does not exist yet
    // NOTE: This only works for files filname + 0-9.csv for some reason
    unsigned int i = 0;
    sprintf(filename, "%s_%d%s", basename, i++, extension);
    FILINFO fi;
    DEBUG2 = DEBUG_ON;
    while (f_stat(filename, &fi) == 0){
        sprintf(filename, "%s_%d%s", basename, i++, extension);
    }
    accessmode= (FA_CREATE_ALWAYS | FA_OPEN_ALWAYS | FA_CREATE_NEW | FA_WRITE);
    DEBUG2 = DEBUG_OFF;

    //Open the file with the access mode defined above
    FOpenRes=f_open(&file1, filename, (BYTE)accessmode);
    
    // Print the column headers
    f_printf(&file1, "Timestamp, Altitude, X Accel (KX222), Y Accel (KX222), Z Accel (KX222)\n");

    // NOTE: This is used if the BNO55 is being used
    //f_printf(&file1, "Timestamp, Altitude, Euler Heading, Euler Pitch, Euler Roll, Gyro X, Gyro Y, Gyro Z, Mag X, Mag Y, Mag Z, X Accel (BNO), Y Accel (BNO), Z Accel (BNO), X Accel (KX222), Y Accel (KX222), Z Accel (KX222)\n");
   
    DEBUG1 = DEBUG_OFF;
}


void SD_write_str(const char* input){
    
    // Writes string to the sd card file. Turns on DEBUG1 while writing
    DEBUG1 = DEBUG_ON;
    
    f_printf(&file1, input);
    
    DEBUG1 = DEBUG_OFF;
}

void SD_write_char(const char input){
    
    // Writes character to the sd card file. Turns on DEBUG1 while writing
    DEBUG1 = DEBUG_ON;
    
    f_putc(input, &file1);
    
    DEBUG1 = DEBUG_OFF;
}

void SD_sync(){
    
    // Sync's the file with the SD card (forces the data to be written to the card)
    // Turns on DEBUG1 while syncing
    DEBUG1 = DEBUG_ON;
    
    f_sync(&file1);
    
    DEBUG1 = DEBUG_OFF;
}

/*
 *  Closes the save file and un-mounts the file system
 */
void SD_unmount(){
    //close the file
    f_close(&file1);
    
    //unmount filesystem
    f_mount(0,NULL);
    
}