#include <xc.h>
#include "i2c1mz.h"

void I2C1_init(unsigned long rate){
    ETHCON1bits.ON = 0;
    
    rate = 100000;
    // Disable interrupts
    IEC3bits.I2C1BIE = 0;
    IEC3bits.I2C1MIE = 0;
    IEC3bits.I2C1SIE = 0;
        
    /*
    // set the clock rate based on user input
    unsigned long pbclk = 25000000; // 25 MHz
    I2C15BRG = pbclk/(2*rate) - 2; // this ignores pulse gobble
    */
    I2C1BRG = 0x060; // PIC32 Reference Section 24

    // turn off for config
    I2C1CONbits.ON = 0;
    
    if(rate < 200000){
        I2C1CONbits.DISSLW = 1;
    }
    else{
        I2C1CONbits.DISSLW = 0;
    }
    
    // turn on after config
    I2C1CONbits.ON = 1;
    MIF1 = 0;    // clear flag
    
}


void I2C1_start(void){
    
    // we can experiment with robustness later
    I2C1CONbits.SEN = 1;                // initialize start
	while(!MIF1){}            // wait until start finished
    MIF1 = 0; // clear flag
 
}


void I2C1_stop(void){
    I2C1CONbits.PEN=1;                  // initiate stop, PEN=1
    while(!MIF1){}            // wait until stop finished
	MIF1 = 0;
}


void I2C1_restart(void){
    I2C1CONbits.RSEN = 1;
	while(!MIF1);
    MIF1 = 0;
}


uint8_t I2C1_write(uint8_t data) {
    I2C1TRN = data;                     // put data in transmit buffer
    while(!MIF1){}        // wait until transfer finished
    MIF1 = 0;
    return(I2C1STATbits.ACKSTAT);       // return ACK         
}


uint8_t I2C1_read(int8_t ack2send) {
    unsigned char inByte;
    //while(I2C15CON & 0x1F){}             // wait for idle condition (nothing in progress)
    
    
    I2C1CONbits.RCEN = 1;               // enable receive
    while(!MIF1){}          // wait for receive byte
    MIF1 = 0;
    inByte = I2C1RCV;                   // read byte
    //while(I2C15CON & 0x1F){}             // wait for idle
    
    I2C1_ACK(ack2send);					// send ACK signal
    return(inByte);						// return received byte
	
}


void I2C1_ACK(int8_t ack2send){
	I2C1CONbits.ACKDT = ack2send;       // ACK type to send
    I2C1CONbits.ACKEN = 1;              // enable ACK bit transmission
    while(!MIF1){}          // wait until done
	MIF1 = 0;
}
