#include <xc.h>
#include "i2c3mz.h" 

void I2C3_init(){
    ETHCON1bits.ON = 0; // disable ethernet to ensure no collisions
    
    // Disable interrupts
    IEC5bits.I2C3BIE = 0;
    IEC5bits.I2C3MIE = 0;
    IEC5bits.I2C3SIE = 0;
        
    // Clock rate hard coded by this data
    I2C3BRG = 0x060; // PIC32 Reference Section 24 - 100kHz
    int rate = 100000;

    // turn off for config
    I2C3CONbits.ON = 0;
    
    if(rate < 200000){
        I2C3CONbits.DISSLW = 1;
    }
    else{
        I2C3CONbits.DISSLW = 0;
    }
    
    // turn on after config
    I2C3CONbits.ON = 1;
    MIF3 = 0;    // clear flag
    
}


void I2C3_start(void){
    
    // we can experiment with robustness later
    I2C3CONbits.SEN = 1;                // initialize start
	while(!MIF3){}            // wait until start finished
    MIF3 = 0; // clear flag
 
}


void I2C3_stop(void){
    I2C3CONbits.PEN=1;                  // initiate stop, PEN=1
    while(!MIF3){}            // wait until stop finished
	MIF3 = 0;
}


void I2C3_restart(void){
    I2C3CONbits.RSEN = 1;
	while(!MIF3);
    MIF3 = 0;
}


uint8_t I2C3_write(uint8_t data) {
    I2C3TRN = data;                     // put data in transmit buffer
    while(!MIF3){}        // wait until transfer finished
    MIF3 = 0;
    return(I2C3STATbits.ACKSTAT);       // return ACK         
}


uint8_t I2C3_read(int8_t ack2send) {
    unsigned char inByte;
    //while(I2C5CON & 0x1F){}             // wait for idle condition (nothing in progress)
    
    
    I2C3CONbits.RCEN = 1;               // enable receive
    while(!MIF3){}          // wait for receive byte
    MIF3 = 0;
    inByte = I2C3RCV;                   // read byte
    //while(I2C5CON & 0x1F){}             // wait for idle
    
    I2C3_ACK(ack2send);					// send ACK signal
    return(inByte);						// return received byte
	
}


void I2C3_ACK(int8_t ack2send){
	I2C3CONbits.ACKDT = ack2send;       // ACK type to send
    I2C3CONbits.ACKEN = 1;              // enable ACK bit transmission
    while(!MIF3){}          // wait until done
	MIF3 = 0;
}
