#include <xc.h>
#include "i2c5mz.h" 

void I2C5_init(){
    ETHCON1bits.ON = 0; // disable ethernet to ensure no collisions
    
    // Disable interrupts
    IEC5bits.I2C5BIE = 0;
    IEC5bits.I2C5MIE = 0;
    IEC5bits.I2C5SIE = 0;
        
    // Clock rate hard coded by this value
    I2C5BRG = 0x060; // PIC32 Reference Section 24 - 100kHz
    int rate = 100000;

    // turn off for config
    I2C5CONbits.ON = 0;
    
    if(rate < 200000){
        I2C5CONbits.DISSLW = 1;
    }
    else{
        I2C5CONbits.DISSLW = 0;
    }
    
    // turn on after config
    I2C5CONbits.ON = 1;
    MIF5 = 0;    // clear flag
    
}


void I2C5_start(void){
    
    // we can experiment with robustness later
    I2C5CONbits.SEN = 1;                // initialize start
	while(!MIF5){}            // wait until start finished
    MIF5 = 0; // clear flag
 
}


void I2C5_stop(void){
    I2C5CONbits.PEN=1;                  // initiate stop, PEN=1
    while(!MIF5){}            // wait until stop finished
	MIF5 = 0;
}


void I2C5_restart(void){
    I2C5CONbits.RSEN = 1;
	while(!MIF5);
    MIF5 = 0;
}


uint8_t I2C5_write(uint8_t data) {
    I2C5TRN = data;                     // put data in transmit buffer
    while(!MIF5){}        // wait until transfer finished
    MIF5 = 0;
    return(I2C5STATbits.ACKSTAT);       // return ACK         
}


uint8_t I2C5_read(int8_t ack2send) {
    unsigned char inByte;
    //while(I2C5CON & 0x1F){}             // wait for idle condition (nothing in progress)
    
    
    I2C5CONbits.RCEN = 1;               // enable receive
    while(!MIF5){}          // wait for receive byte
    MIF5 = 0;
    inByte = I2C5RCV;                   // read byte
    //while(I2C5CON & 0x1F){}             // wait for idle
    
    I2C5_ACK(ack2send);					// send ACK signal
    return(inByte);						// return received byte
	
}


void I2C5_ACK(int8_t ack2send){
	I2C5CONbits.ACKDT = ack2send;       // ACK type to send
    I2C5CONbits.ACKEN = 1;              // enable ACK bit transmission
    while(!MIF5){}          // wait until done
	MIF5 = 0;
}
